proc name2id {a} {
    if { [regexp -- {[0-9]} [string range $a 0 0]]} { return $a }
    return $::TAC(rooms,id,$a)
}

proc userpref {a} {
    if {$::TAC(nameid) == 1} {
        return $::TAC(rooms,name,$a)
    } elseif {$::TAC(nameid) == 0} {
        return $a
    } else {
        return "$::TAC(rooms,name,$a) (id $a)"
    }
}

proc CHAT_INVITE {name loc id sender msg} {
    tac_output "$sender invited you to chat room '$loc' with id '$id'"
    tac_output "Invitation message is '$msg'"
    tac_output "To accept, type /ca $id or c:a $id"
}

proc CHAT_UPDATE_BUDDY {name id inside blist} {
     foreach i $blist {
         if {$inside == "T"} {
             tac_output "[ts c]** $i has entered [userpref $id]."
             lappend ::TAC(rooms,who,$id) $i
         } else {
             tac_output "[ts c]** $i has left $::TAC(rooms,name,$id)."
             set x [lsearch $::TAC(rooms,who,$id) $i]
             set ::TAC(rooms,who,$id) [lreplace $::TAC(rooms,who,$id) $x $x]
         }
     }
}

proc CHAT_JOIN {name id loc} {
    tac_output "You have joined chat room '$loc' with id '$id'."
    lappend ::TAC(rooms,id) $id
    lappend ::TAC(rooms,name) $loc
    set ::TAC(rooms,name,$id) $loc
    set ::TAC(rooms,id,$loc) $id
}

proc CHAT_LEFT {name id} {
    tac_output "You have left chat room [userpref $id]"
    set x [lsearch $::TAC(rooms,id) $id]
    set ::TAC(rooms,id) [lreplace $::TAC(rooms,id) $x $x]
}

proc CHAT_IN {name id source whisper msg} {
    tac_receive_chat $id $source $whisper $msg
}

proc tac_receive_chat {id remote whisper msg} {
    set whisperstr ""
    if { $whisper == "T" } {
        set whisperstr " (whispers) "
    }

    set text [unHTML $msg] 
    tac_output "<[userpref $id]:$remote>$whisperstr $text"
}

proc tac_cmd_chat_accept {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([0-9]+)$} $line match cmd id]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set id [name2id $id]
    toc_chat_accept $::NSCREENNAME $id
}

proc tac_cmd_chat_invite {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([a-zA-Z0-9]+) ([a-zA-Z0-9]+) (.*)$} $line match cmd room user msg} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set id [name2id $room]
    toc_chat_invite $::NSCREENNAME $id "$msg" $user
}

proc tac_cmd_chat_msg {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) +([a-zA-Z0-9]+) (.*)$} $line match cmd room msg]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set id [name2id $room]
    toc_chat_send $::NSCREENNAME $id $msg
    set ::TAC(lastchat) $id
}

proc tac_cmd_chat_evil {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([a-zA-Z0-9]+) ([a-zA-Z0-9]+) *$} $line match cmd room name]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set id [name2id $room]
    toc_chat_evil $::NSCREENNAME $id $name
}

proc tac_cmd_chat_who {exp line} {
    if {[string range $exp end end] == "$"} {
        foreach i $::TAC(rooms,id) {
            tac_output "[userpref $i]"
            foreach j $::TAC(rooms,who,$i) {
                tac_output "  $j"
            }
        }
        return
    }
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([a-zA-Z0-9]+) *$} $line match cmd room]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

# this list is maintained by the client, not the server.
    set id [name2id $room]
    if {$::TAC(rooms,who,$id) == [list]} { return }
    foreach i $::TAC(rooms,who,$id) {
        tac_output $i
    }
}

proc tac_cmd_chat_join {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([a-zA-Z0-9]+)$} $line match cmd room]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_chat_join $::NSCREENNAME 4 $room
}

proc tac_cmd_chat_leave {exp line} {
    if {[string range $exp end end] == "$"} {
        foreach i $::TAC(rooms,id) {
            toc_chat_leave $::NSCREENNAME $i
        }
        return
    }
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([a-zA-Z0-9]+)$} $line match cmd room]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set id [name2id $room]
    toc_chat_leave $::NSCREENNAME $id
}

proc tac_cmd_defleave {exp line} {
    if {![regexp -- {^\@?c[0-9][0-9]?\@?$} $line match cmd]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set cmd $line
    if {[string range $cmd 0 0]=="@"} {
        set cmd [string range $cmd 1 end]
    } elseif {[string range $cmd end end]=="@"} {
        set cmd [string range $cmd 0 [expr [string length $cmd] - 2]]
    } else {
        tac_output "Usage: [tac_get_help $exp]"
        tac_output "DEBUG: slipped through regexp"
    }
    set id [name2id $::TAC(defchat,$cmd)]
    toc_chat_leave $::NSCREENNAME $id
}

proc tac_cmd_chat_whisper {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([a-zA-Z0-9]+) ([a-zA-Z0-9]+) (.*)$} $line match cmd room user msg} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }


    set id [name2id $room]
    toc_chat_whisper $::NSCREENNAME $id $user $msg
    set ::TAC(lastchat) $id
    log_chat_whisper $id $user $msg
}

proc tac_cmd_chat_list {exp line} {
    foreach i $::TAC(rooms,id) {
        tac_output [userpref $i]
    }
}

toc_register_func * CHAT_INVITE       CHAT_INVITE
toc_register_func * CHAT_JOIN         CHAT_JOIN
toc_register_func * CHAT_LEFT         CHAT_LEFT
toc_register_func * CHAT_UPDATE_BUDDY CHAT_UPDATE_BUDDY
toc_register_func * CHAT_IN           CHAT_IN

tac_register_cmd 1 "c" "^/ca "      tac_cmd_chat_accept    "  /ca id           accept chat id"
tac_register_cmd 1 "c" "^/cm "      tac_cmd_chat_msg       "  /cm room msg     send msg to the chat room"
tac_register_cmd 1 "c" "^/ce "      tac_cmd_chat_evil      "  /ce room user    warn a user in a chat room"
tac_register_cmd 1 "c" "^/cw "      tac_cmd_chat_who       "  /cw {room}       show users in a chat room"
tac_register_cmd 1 "c" "^/cw$"      tac_cmd_chat_who
tac_register_cmd 1 "c" "^/cj "      tac_cmd_chat_join      "  /cj name         join chat room 'name'"
tac_register_cmd 1 "c" "^/cs "      tac_cmd_chat_whisper   "  /cs rm user msg  whisper to user in chat room"
tac_register_cmd 1 "c" "^/ci "      tac_cmd_chat_invite    "  /ci rm user msg  invite user to a chat room"
tac_register_cmd 1 "c" "^/cl "      tac_cmd_chat_leave     "  /cl {room}       leave a chat room (or all rooms)"
tac_register_cmd 1 "c" "^/cl$"      tac_cmd_chat_leave
tac_register_cmd 1 "c" "^/cr$"      tac_cmd_chat_list      "  /cr              list chat rooms you are in"

tac_register_cmd 2 "c" "^c:a "      tac_cmd_chat_accept    "  c:a id           accept chat id"
tac_register_cmd 2 "c" "^c:m "      tac_cmd_chat_msg       "  c:m room msg     send msg to the chat room"
tac_register_cmd 2 "c" "^c:e "      tac_cmd_chat_evil      "  c:e room user    warn a user in a chat room"
tac_register_cmd 2 "c" "^c:w "      tac_cmd_chat_who       "  c:w {room}       show users in a chat room"
tac_register_cmd 2 "c" "^c:w$"      tac_cmd_chat_who
tac_register_cmd 2 "c" "^c:j "      tac_cmd_chat_join      "  c:j name         join chat room 'name'"
tac_register_cmd 2 "c" "^c:s "      tac_cmd_chat_whisper   "  c:s rm user msg  whisper to user in chat room"
tac_register_cmd 2 "c" "^c:i "      tac_cmd_chat_invite    "  c:i rm user msg  invite user to a chat room"
tac_register_cmd 2 "c" "^c:l "      tac_cmd_chat_leave     "  c:l {room}       leave a chat room (or all rooms)"
tac_register_cmd 2 "c" "^c:l$"      tac_cmd_chat_leave
tac_register_cmd 2 "c" "^c:r$"      tac_cmd_chat_list      "  c:r              list chat rooms you are in"
tac_register_cmd 2 "c" "^c:h$"      tac_cmd_help

set TAC(rooms,id) [list]
set TAC(rooms,name) [list]
set TAC(nameid) 1         ;# this means name
