proc info_CONFIG {name data} {
    if { $::TAC(initinfo) != "" } { toc_set_info $::NSCREENNAME $::TAC(initinfo) }
}

proc info_GOTO_URL {name window url} {
    if {$::TAC(urlstat) != 1} { return }
    set ::TAC(urlstat) 0
    if {[lsearch [package names] http] == -1} {
        tac_output "Go to this URL: $url"
        return
    }

    if {![string match "http://*" $url]} {
        set ip [lindex [sflap::peerinfo $name] 0]
        set url "http://$ip/$url"
    }

    if {[string match "http://*" $url]} {
        set tokn [::http::geturl $url]
        upvar #0 $tokn state
        infoHTML $state(body)
    } else {
        infoHTML $url
    }
}

proc infoHTML {msg} { 
    set x 0
    set msg [string range $msg 1 [expr [string last "\<hr\>" [string tolower $msg]] - 1]]
    if {[string match "*aol_icon.gif*" $msg]} { tac_output "AOL User" }
    if {[string match "*free_icon.gif*" $msg]} { tac_output "AIM User" }
    if {[string match "*dt_icon.gif*" $msg]} { tac_output "Trial User" }
    if {[string match "*admin_icon.gif*" $msg]} { tac_output "Administrator" }
    set msg [string range $msg [expr [string first "\<hr\>" [string tolower $msg]] - 1] end]

    foreach e [splitHTML $msg] {
        if {![regexp -- {^<.*>$} $e match tag]} {
            set e [string trim $e]
            if {[string length $e]} {
                tac_output $e
                set x 1
            }
        }
    }
    if {$x == 0} { 
        tac_output "User information is blank." 
    }
    tac_output "----"
}

proc tac_cmd_ginfo {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) +([a-zA-Z0-9]+) *$} $line match cmd name]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    set ::TAC(urlstat) 1
    tac_output "User Information for $name"
    toc_get_info $::NSCREENNAME $name
}

proc tac_cmd_sinfo {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) (.*)$} $line match cmd message]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_set_info $::NSCREENNAME $message
    tac_output "Info set."
}

proc tac_set_info {info} {
    set ::TAC(initinfo) $info
}

proc tac_cmd_idle {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) ([0-9]+)$} $line match cmd secs]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_set_idle $::NSCREENNAME $secs
}
   
tac_register_cmd 1 "i" "^/ig "       tac_cmd_ginfo           "  /ig user         get info on user"
tac_register_cmd 1 "i" "^/is "       tac_cmd_sinfo           "  /is message      set your own user info"
tac_register_cmd 1 "i" "^/it "       tac_cmd_idle            "  /it secs         set idle time"

tac_register_cmd 2 "i" "^i:g "       tac_cmd_ginfo           "  i:g user         get info on user"
tac_register_cmd 2 "i" "^i:s "       tac_cmd_sinfo           "  i:s message      set your own user info"
tac_register_cmd 2 "i" "^i:t "       tac_cmd_idle            "  i:t secs         set idle time"
tac_register_cmd 2 "i" "^i:h$"       tac_cmd_help

toc_register_func * GOTO_URL info_GOTO_URL
toc_register_func * CONFIG info_CONFIG

set ::TAC(initinfo) ""
