#! /bin/sh
# The next line is executed by /bin/sh, but not Tcl \
exec tclsh $0 ${1+"$@"}

# TAC.TCL --
# A Tcl only version of AOL Instant Messenger that has a IRC ish
# interface. Because this is in Tcl and doesn't use curses, the line
# editing doesn't work very well on some platforms.
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Bugs: smike@users.tmok.com
# Home Page: http://users.tmok.com/~smike

set VERSION "Tac 0.17"
set REVISION {TAC:$Revision: 0.17 $}

# Make sure we are in the tac directory and load the toc routines.
cd [file dirname $argv0]
catch {cd [file dirname [file readlink $argv0]]}
source toc.tcl

set TAC(input) 0
set TAC(output) ""

proc unHTML {msg} {
# ugh. aim is in HTML. we hates it my precious.
    foreach e [splitHTML $msg] {
        if {![regexp -- {^<.*>$} $e match tag]} {
            append text $e
        }
    }
    return $text
}

#******************************************************
#*******************INCOMING PROTOCOL******************
#******************************************************

proc SIGN_ON {name version} {
    # The following is true after migration
    if {[llength $::BUDDYLIST] > 0} {  
        tac_send_init 1
    }

    tac_output "...Welcome to AOL Instant Messenger ($::VERSION)..."
}

proc CONFIG {name data} {
    catch { tac_parse_config $data }
    tac_send_init 1
}

proc NICK {name nick} {
    set ::SCREENNAME $nick
}


proc IM_IN {name source msg auto} {
    set text [unHTML $msg]
    tac_output_nnl "[ts x]<*$source*>"
    if { $auto } { tac_output_nnl " (away)" }
    tac_output " $text"
}

proc UPDATE_BUDDY {name user online evil signon idle uclass} {
    set bud [normalize $user]
    set ::BUDDIES($bud,name) $user
    set o $::BUDDIES($bud,online)
    set ::BUDDIES($bud,online) $online
    set ::BUDDIES($bud,evil) $evil
    set ::BUDDIES($bud,signon) $signon
    set i $::BUDDIES($bud,idle)
    set ::BUDDIES($bud,idle) $idle
    if { ($o != $online) } {
        if {$online == "T"} {
            tac_output "[ts b]** $user has signed on."
        } else {
            tac_output "[ts b]** $user has signed off."
        }
    }
}

proc ERROR {name code data} {
    set args [split $data ":"]
    switch -exact -- $code {
    "901" { tac_output "[lindex $args 0] not currently available." }
    "902" { tac_output "Warning of [lindex $args 0] is not allowed." }
    "903" {
        tac_output "A message has been dropped, you\
        are exceeding the server speed limit."
    }
    "950" { tac_output "Chat in [lindex $args 0] is unavailable." }
    "960" {
        tac_output \
            "You are sending message too fast to [lindex $args 0]."
    }
    "961" {
        tac_output \
            "You missed an im from [lindex $args 0] because it was too big."
    }
    "962" {
        tac_output "You missed an im from\
                               [lindex $args 0] because it was sent too fast."
    }
    "970" { tac_output "Failure." }
    "971" { tac_output "Too many matches." }
    "972" { tac_output "Need more qualifiers." }
    "973" { tac_output "Dir service temporarily unavailable." }
    "974" { tac_output "Email lookup restricted." }
    "975" { tac_output "Keyword Ignored [lindex $args 0]" }
    "976" { tac_output "No Keywords." }
    "977" { tac_output "Language not supported." }
    "978" { tac_output "Country not supported." }
    "979" { tac_output "Failure unknown: [lindex $args 0]" }
    "980" {
        tac_output "Incorrect nickname or password."
        tac_set_terminal cooked
        exit 1
    }
    "981" { tac_output "The service is temporarly unavailable." }
    "982" {
        tac_output \
            "Your warning level is currently too high to sign on."
        tac_set_terminal cooked
        exit 1
    }
    "983" {
        tac_output "You have been connecting and\
            disconnecting too frequently.  Wait 10 minutes and try again.\
            If you continue to try, you will need to wait even longer."
        tac_set_terminal cooked
        exit 1
    }
    "989" {
        tac_output \
            "An unknown signon error has occured. ([lindex $args 0])"
    }
    "911" { tac_output "Error validating input" }
    "912" { tac_output "Invalid account" }
    "913" { tac_output "Error encountered while processing request" }
    "914" { tac_output "Service unavailable" }

    default { tac_output "Unknown error $code:$data" }
    } ;# SWITCH
}

proc EVILED {name level user} {
    if {[string length $user] == 0 } {
        tac_output \
            "You were warned anonymously! Your new warning level is $level%."
    } else {
        tac_output \
            "You were warned by $user! Your new warning level is $level%."
    }
}

proc GOTO_URL {name window url} {
}

proc PAUSE {name data} {
    tac_output "PAUSING"
}

proc CONNECTION_CLOSED {name data} {
    tac_output "Connection Closed"
    exit 0
}

#******************************************************
#********************** TAC METHODS *******************
#******************************************************

proc tac_parse_config {data} {
    set ::BUDDYLIST [list]
    set ::PERMITLIST [list]
    set ::DENYLIST [list]
    set ::PDMODE 1

    set ::TAC(config) $data

    set lines [split $data "\n"]
    foreach i $lines {
        switch -exact -- [string index $i 0] {
        "b" {
            set bud [normalize [string range $i 2 end]]
            set ::BUDDIES($bud,online) F
            set ::BUDDIES($bud,name) $bud
            set ::BUDDIES($bud,idle) 0
            lappend ::GROUPS($group,people) $bud
        } 
        "d" {
            set deny [string range $i 2 end]
            lappend ::DENYLIST $deny
        }
        "g" {
            set group [string range $i 2 end]
            lappend ::BUDDYLIST $group
            lappend ::GROUPS($group,collapsed) F
            set ::GROUPS($group,people) [list]
        }
        "m" {
            set ::PDMODE [string range $i 2 end]
        }
        "p" {
            set permit [string range $i 2 end]
            lappend ::PERMITLIST $permit
        }
        }
    }

    tac_set_config
}

proc tac_output {str} {
    if {$::TAC(input)} {
        append ::TAC(output) $str
        append ::TAC(output) "\r\n"
    } else {
        puts -nonewline "$str\r\n"
    }
}

proc tac_output_nnl {str} {
    if {$::TAC(input)} {
        append ::TAC(output) $str
    } else {
        puts -nonewline "$str"
    }
}

proc tac_set_config {} {
    set str ""
    append str "m $::PDMODE\n"
    foreach p $::PERMITLIST {
        append str "p $p\n"
    }
    foreach d $::DENYLIST {
        append str "d $d\n"
    }
    foreach g $::BUDDYLIST {
        append str "g $g\n"
        foreach b $::GROUPS($g,people) {
            append str "b $b\n"
        }
    }

    toc_set_config $::NSCREENNAME $str
}

proc tac_send_init {first} {
    foreach g $::BUDDYLIST {
        foreach b $::GROUPS($g,people) {
            lappend buds $b
        }
    }

    if {[info exists buds] == 0} {
        tac_add_buddy Buddies [normalize $::SCREENNAME]
    } else {
        toc_add_buddy $::SCREENNAME $buds
    }

    if {$::PDMODE == "3"} {
        toc_add_permit $::SCREENNAME $::PERMITLIST
    } elseif {$::PDMODE == "4"} {
        toc_add_deny $::SCREENNAME $::DENYLIST
    }

    if {$first == 1} {
        toc_init_done $::SCREENNAME
    }
}

proc tac_connect {} {
    set ::BUDDYLIST [list]
    set ::PERMITLIST [list]
    set ::DENYLIST [list]
    set ::PDMODE 1

    if {![info exists ::SCREENNAME]} {
        puts -nonewline "login: "
        flush stdout
        set ::SCREENNAME [string trimright [gets stdin]]
    }
    set ::NSCREENNAME [normalize $::SCREENNAME]
    if {![info exists ::PASSWORD]} {
        puts -nonewline "Password: "
        flush stdout
        exec stty -echo
        set ::PASSWORD [string trimright [gets stdin]]
        exec stty echo
        puts ""
    }
    flush stdout
    toc_open $::NSCREENNAME toc.oscar.aol.com 5190 login.oscar.aol.com \
             5190 $::NSCREENNAME $::PASSWORD english $::REVISION
    unset ::PASSWORD
}

proc tac_set_terminal {mode} {
    if {$mode == "cooked"} {
        fconfigure stdin -blocking true
        fconfigure stdin -buffering line
        fileevent stdin readable {}
        exec stty cooked
    } elseif {$mode == "raw"} {
        fconfigure stdin -blocking false
        fconfigure stdin -buffering none
        exec stty raw
        fileevent stdin readable tac_handle_input
    }
}

proc tac_register_cmd {syntax package cmdexp function {help {}}} {
    lappend ::TAC(commands) $cmdexp
    set ::TAC(commands,$cmdexp,func) $function
    set ::TAC(commands,$cmdexp,package) $package
    set ::TAC(commands,$cmdexp,syntax) $syntax
    set ::TAC(commands,$cmdexp,help) $help
}

proc tac_unregister_cmd {cmdexp} {
    set pos [lsearch -exact $::TAC(commands) $cmdexp]
    set ::TAC(commands) [lreplace $::TAC(commands) $pos $pos]
    unset ::TAC(commands,$cmdexp,func)
    unset ::TAC(commands,$cmdexp,package)
    unset ::TAC(commands,$cmdexp,syntax)
    unset ::TAC(commands,$cmdexp,help)
}

proc tac_get_help {cmdexp} {
    return $::TAC(commands,$cmdexp,help)
}

proc tac_get_help1 {package syntax} {
    set X1 0
    set packagelist [list a b B c d f h i l o p s t]
    if {$package == ""} {
        foreach cmdexp $::TAC(commands) {
            if {$::TAC(commands,$cmdexp,help) != ""} {
                if {$::TAC(commands,$cmdexp,syntax) != $syntax} {
                    puts $::TAC(commands,$cmdexp,help)
                    set X1 [expr $X1 + 1]
                }
            }
         }
    } elseif {[lsearch $packagelist $package] != -1} {
        foreach cmdexp $::TAC(commands) {
            if {$::TAC(commands,$cmdexp,help) != ""} {
                if {$::TAC(commands,$cmdexp,syntax) != $syntax} {
                    if {$::TAC(commands,$cmdexp,package) == $package} {
                        tac_output $::TAC(commands,$cmdexp,help)
                        set X1 [expr $X1 + 1]
                    }
                }
            }
        }
    } elseif {[string length $package] > 1} {
        tac_output "Invalid package. Package must be one letter."
    } else {
        tac_output "Package must be one of: $packagelist"
    }
}


proc tac_handle_input {} {
    set ::TAC(input) 1
    tac_set_terminal cooked
    set line [string trimright [gets stdin]]

    set found 0
    foreach cmdexp $::TAC(commands) {
        if {[regexp -nocase -- $cmdexp $line]} {
            set found 1
            $::TAC(commands,$cmdexp,func) $cmdexp $line 
        }
    }

    if {$line == ""} { set found 1 }

    if {!$found} {
        tac_cmd_default_action $cmdexp $line
    }

    if {$::TAC(output) != ""} {
        puts -nonewline "$::TAC(output)"
        set ::TAC(output) ""
    }
    set ::TAC(input) 0

    tac_set_terminal raw
}

proc tac_im_wrapper {name msg auto} {
    if { $::TAC(log) == 1 } { tac_log_outIM $name $msg $auto }
    tac_broadcast_outIM $name $msg $auto
}

#******************************************************
#********************USER COMMANDS*********************
#******************************************************

proc tac_cmd_default_action {exp line} {
    tac_output ""
    tac_output "Unknown command $line"
    tac_output "Type /h or :h for a list of valid commands."
    tac_output ""
}

proc tac_cmd_who {exp line} {
    tac_output "  User\t\t\tIdle\tEvil"
    tac_output "------------------------------------"
    foreach g $::BUDDYLIST {
        set ::GROUPS($g,online) 0
        set ::GROUPS($g,total) 0
        foreach b $::GROUPS($g,people) {
           incr ::GROUPS($g,total)
           set o $::BUDDIES($b,online)
           if { ($o == "T") } {
                incr ::GROUPS($g,online)
           }
        } 
        tac_output_nnl $g
        if {[string length $g] < 8} {
            tac_output "\t\t($::GROUPS($g,online)/$::GROUPS($g,total))" 
           } else {
            tac_output "\t($::GROUPS($g,online)/$::GROUPS($g,total))" 
        }
        foreach b $::GROUPS($g,people) {
            set o $::BUDDIES($b,online)
            if { ($o == "T") } { 
                set out "  $::BUDDIES($b,name)\t"
                if {[string length $::BUDDIES($b,name)] < 6} {
                    append out "\t\t"
                } elseif {[string length $::BUDDIES($b,name)] < 14} {
                    append out "\t"
                }
                tac_output "$out$::BUDDIES($b,idle)\t$::BUDDIES($b,evil)%"
            }
        }
#        tac_output "\t\t($::GROUPS($g,online)/$::GROUPS($g,total))"
    }
}

proc tac_cmd_im {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) +([a-zA-Z0-9]+) (.*)$} $line match cmd name msg]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_send_im $::NSCREENNAME $name $msg
    tac_im_wrapper $name $msg 0
}

proc tac_cmd_evil {exp line} {
    if {![regexp -- {^([a-zA-Z0-9/:]+) +([a-zA-Z0-9]+) *$} $line match cmd name]} {
        tac_output "Usage: [tac_get_help $exp]"
        return
    }

    toc_evil $::NSCREENNAME $name
}

proc tac_cmd_quit {exp line} {
    set ::DONE 1
}

proc tac_cmd_helpline {exp line} {
    set package [string range $line 3 end]
    set x [string range $exp 1 1]
    if {$x == "/"} {
        set syntax 2
    } elseif {$x == ":"} {
        set syntax 1
    } else { return }
    tac_get_help1 $package $syntax
}

proc tac_cmd_help {exp line} {
    set exp [string range $exp 1 [expr [string length $exp] - 2]]
    if {$exp == "g"} {set exp "b"}
    set pack [string range $exp 0 0]

    foreach cmdexp $::TAC(commands) {
        if {$::TAC(commands,$cmdexp,help) != ""} {
            if {$::TAC(commands,$cmdexp,package) == $pack} {
                if {$::TAC(commands,$cmdexp,syntax) != 1} {
                    tac_output $::TAC(commands,$cmdexp,help)
                }
            }
        }
    }

}

#******************************************************
#********************** MAIN **************************
#******************************************************

catch { package require http 2.0 }

toc_register_func * SIGN_ON           SIGN_ON
toc_register_func * CONFIG            CONFIG
toc_register_func * NICK              NICK
toc_register_func * IM_IN             IM_IN
toc_register_func * UPDATE_BUDDY      UPDATE_BUDDY
toc_register_func * ERROR             ERROR
toc_register_func * EVILED            EVILED
toc_register_func * GOTO_URL          GOTO_URL
toc_register_func * PAUSE             PAUSE
toc_register_func * CONNECTION_CLOSED CONNECTION_CLOSED

tac_register_cmd 0 "" "^exit$"     tac_cmd_quit
tac_register_cmd 0 "" "^quit$"     tac_cmd_quit
tac_register_cmd 0 "" "^bye$"      tac_cmd_quit

tac_register_cmd 1 "" "^/h "       tac_cmd_helpline       "  /h {package}     get list of valid commands"
tac_register_cmd 1 "" "^/h$"       tac_cmd_helpline
tac_register_cmd 1 "" "^/m "       tac_cmd_im             "  /m user message  send an im"
tac_register_cmd 1 "" "^/e "       tac_cmd_evil           "  /e user          warn a user"
tac_register_cmd 1 "" "^/w$"       tac_cmd_who            "  /w               show currently logged in buddies"
tac_register_cmd 1 "" "^/q$"       tac_cmd_quit           "  /q               log out"

tac_register_cmd 2 "" "^:h "       tac_cmd_helpline       "  :h {package}     get list of valid commands"
tac_register_cmd 2 "" "^:h$"       tac_cmd_helpline
tac_register_cmd 2 "" "^:m "       tac_cmd_im             "  :m user message  send an im"
tac_register_cmd 2 "" "^:e "       tac_cmd_evil           "  :e user          warn a user"
tac_register_cmd 2 "" "^:w$"       tac_cmd_who            "  :w               show currently logged in buddies"
tac_register_cmd 2 "" "^:q$"       tac_cmd_quit           "  :q               log out"

# Globals and Options
set TAC(rcfile) ~/.tacrc
set TAC(urlstat) 0
set TAC(morestring) "--MORE--"
set TAC(pagerlen) 23

# Set Terminal to normal state
fconfigure stdin -translation binary
tac_set_terminal cooked

# Parse Options
proc argCheck {arg i l} {
    if {$i >= $l} {
        puts "$::argv0: Missing argument for $arg, try -H for usage."
        exit 1;
    }
}
set largv [llength $argv]
for {set i 0} {$i < $largv} {incr i} {
    set arg [lindex $argv $i]
    switch -glob -- $arg {
    "-H" -
    "-h" -
    "-help" {
        puts "$argv0 Usage:"
        puts " -H             This message"
        puts " -sflap <level> SFLAP debug level"
        puts " -rcfile <fn>   Use <fn> instead of $::TAC(rcfile)"
        puts " -user <user>   Use <user>, overrides rc files"
        puts " -pass <pass>   User <pass>, overrides rc files"
        puts " -roast <pass>  Roast a password for use in ~/.tacrc or -pass"
        exit 0
    }
    "-sflap" {
        incr i
        argCheck $arg $i $largv
        set sflap::debug_level [lindex $argv $i]
    }
    "-rcfile" {
        incr i
        argCheck $arg $i $largv
        set ::TAC(rcfile) [lindex $argv $i]
    }
    "-u" -
    "-user" {
        incr i
        argCheck $arg $i $largv
        set SCREENNAME [set ::TAC(clUser) [lindex $argv $i]]
    }
    "-p" - 
    "-pass" {
        incr i
        argCheck $arg $i $largv
        set PASSWORD [set ::TAC(clPass) [lindex $argv $i]]
    }
    "-roast" {
        incr i
        argCheck $arg $i $largv
        puts -nonewline "\nPlace \"set PASSWORD "
        puts "[roast_password [lindex $argv $i]]\" in your $::TAC(rcfile) file.\n"
        puts -nonewline "WARNING: While not in clear text, people can still "
        puts "decode your password."
        puts "You should do a \"chmod og-rwx $::TAC(rcfile)\" for some protection."
        exit 0
    }
    default {
        puts "$::argv0: Unknown argument '$arg', try -H for usage."
    }
    } ;# SWITCH
}

# Load packages
set files [glob -nocomplain -- packages/*.tcl]
foreach pkg $files {
  set b 0
  if {$pkg == "packages/shell.tcl" } { exec rm -f packages/shell.tcl; set b 1 }
  if {$pkg == "packages/ver.tcl" } { exec rm -f packages/ver.tcl; set b 1 }
  if { $b == 0 } { source $pkg }
}

# Load the user config
if {![file exists $::TAC(rcfile)]} {
    set i [open example.tacrc r]
    set o [open $::TAC(rcfile) w]
    puts $o [read $i]
    close $o
    close $i
}
source $::TAC(rcfile)

tac_connect

tac_set_terminal raw

vwait DONE

if { $::TAC(log) == 1 && $::TAC(logwhat,session) == 1 } {
    set fn [open $::TAC(logfile) a]
    puts $fn "=== Log end [clock format [clock seconds] -format "%A, %B %d %Y %I:%M:%S%p"] ==="
    puts $fn ""
    close $fn
}

tac_set_terminal cooked
exit 0
